import pretrainedmodels

from .change_last_linear import cadene_change_last_linear

lookup = {
    "fbresnet152": pretrainedmodels.models.fbresnet.fbresnet152,
    "bninception": pretrainedmodels.models.bninception,
    "resnext101_32x4d": pretrainedmodels.models.resnext.resnext101_32x4d,
    "resnext101_64x4d": pretrainedmodels.models.resnext.resnext101_64x4d,
    "inceptionv4": pretrainedmodels.models.inceptionv4,
    "inceptionresnetv2": pretrainedmodels.models.inceptionresnetv2,
    "alexnet": pretrainedmodels.models.torchvision_models.alexnet,
    "densenet121": pretrainedmodels.models.torchvision_models.densenet121,
    "densenet169": pretrainedmodels.models.torchvision_models.densenet169,
    "densenet201": pretrainedmodels.models.torchvision_models.densenet201,
    "densenet161": pretrainedmodels.models.torchvision_models.densenet161,
    "resnet18": pretrainedmodels.models.torchvision_models.resnet18,
    "resnet34": pretrainedmodels.models.torchvision_models.resnet34,
    "resnet50": pretrainedmodels.models.torchvision_models.resnet50,
    "resnet101": pretrainedmodels.models.torchvision_models.resnet101,
    "resnet152": pretrainedmodels.models.torchvision_models.resnet152,
    "inceptionv3": pretrainedmodels.models.torchvision_models.inceptionv3,
    "squeezenet1_0": pretrainedmodels.models.torchvision_models.squeezenet1_0,
    "squeezenet1_1": pretrainedmodels.models.torchvision_models.squeezenet1_1,
    "vgg11": pretrainedmodels.models.torchvision_models.vgg11,
    "vgg11_bn": pretrainedmodels.models.torchvision_models.vgg11_bn,
    "vgg13": pretrainedmodels.models.torchvision_models.vgg13,
    "vgg13_bn": pretrainedmodels.models.torchvision_models.vgg13_bn,
    "vgg16": pretrainedmodels.models.torchvision_models.vgg16,
    "vgg16_bn": pretrainedmodels.models.torchvision_models.vgg16_bn,
    "vgg19_bn": pretrainedmodels.models.torchvision_models.vgg19_bn,
    "vgg19": pretrainedmodels.models.torchvision_models.vgg19,
    "nasnetamobile": pretrainedmodels.models.nasnet_mobile.nasnetamobile,
    "nasnetalarge": pretrainedmodels.models.nasnet.nasnetalarge,
    "dpn68": pretrainedmodels.models.dpn.dpn68,
    "dpn68b": pretrainedmodels.models.dpn.dpn68b,
    "dpn92": pretrainedmodels.models.dpn.dpn92,
    "dpn98": pretrainedmodels.models.dpn.dpn98,
    "dpn131": pretrainedmodels.models.dpn.dpn131,
    "dpn107": pretrainedmodels.models.dpn.dpn107,
    "xception": pretrainedmodels.models.xception,
    "senet154": pretrainedmodels.models.senet.senet154,
    "se_resnet50": pretrainedmodels.models.senet.se_resnet50,
    "se_resnet101": pretrainedmodels.models.senet.se_resnet101,
    "se_resnet152": pretrainedmodels.models.senet.se_resnet152,
    "se_resnext50_32x4d": pretrainedmodels.models.senet.se_resnext50_32x4d,
    "se_resnext101_32x4d": pretrainedmodels.models.senet.se_resnext101_32x4d,
    "cafferesnet101": pretrainedmodels.models.cafferesnet.cafferesnet101,
    "pnasnet5large": pretrainedmodels.models.pnasnet.pnasnet5large,
    "polynet": pretrainedmodels.models.polynet,
    "cadene_change_last_linear": cadene_change_last_linear,
}
