from .core import *

lookup = {
    "unet_with_resnet18_backbone": unet_with_resnet18_backbone,
    "unet_with_resnet34_backbone": unet_with_resnet34_backbone,
    "unet_with_resnet50_backbone": unet_with_resnet50_backbone,
    "unet_with_resnet101_backbone": unet_with_resnet101_backbone,
    "unet_with_resnet152_backbone": unet_with_resnet152_backbone,
}
